/*************************************************************************************************************
/ IEC60870 RTU
/ Copyright by Panasonic Electric Works Europe AG 2003-2013
/ RT 27.02.2012
**************************************************************************************************************/

//Global variables for plc parameters
var m_AddressBuffer="DT11100";	//Full PLC address of first m_AddressBuffer register
var m_AddressBufferDT;									//First m_AddressBuffer address (DT area) calculated from m_AddressBuffer
var m_bIsAreaDT;											//m_AddressBuffer in DT or FL area
var m_iError = 0;											//Ajax communication error

//Global variables / parameters
var m_iNumberOfRecords = 8;							//History: point of records to load in one request
var m_iSizeOfRecord = 3;								//History: point of data per record
var m_iMaxLines = 800;									//Max lines to display
var m_sAlternativeColor = "#1362AD";				//Second text color

//Global variables for displaying the plc data
var m_sType;
var m_sIOA;
var m_sStatus;
var m_sNumber;
var m_bColor;
var m_iLastType;
var m_iLastIOA;

//Global variables for interval handling
var m_interval = 500;					//Main interval to update the data
var m_iLoopInterval;						//Checks the transfer status and start the next download of data
var m_iWait = 1;							//Wait before repeat loading data
var m_iDataLoop;							//Selection of the trend line
var m_iLines;								//Lines to display
var m_bDownloadOK;						//Handshake for data download
var m_iNotFirstTime;						//Work different in the first cycle
var zeroes="00000";						//For generating leading zeroes

//System global 
var PLCreg = new Array(m_iNumberOfRecords*m_iSizeOfRecord);  //Response data array (16 bit words or integers)
var XML_Data;								//Buffer for MEWTOCOL request command
var PLC_StationAddress = "EE";		//PLC MEWTOCOL station address ("EE", "01"..."64")


//JavaScript started on page load
window.onload = function()
{
	m_bDownloadOK = 1;			//Start download
	m_iNotFirstTime = 0;			//At first start update immediately, later in one step
	m_iLoopInterval = 0;			//Task manager
	m_iDataLoop = 0;				//Data block selector
	m_iLines = m_iMaxLines-2;
	ResetTags();					//Clear the message area

	m_AddressBufferDT = parseInt(m_AddressBuffer.substring(2,m_AddressBuffer.length));	//String "DT90054" --> 90054
	m_bIsAreaDT = m_AddressBuffer.indexOf("DT");										// FL is -1
	XML_Data = GenerateRD(PLC_StationAddress, m_AddressBufferDT, m_iNumberOfRecords*m_iSizeOfRecord, m_bIsAreaDT);	//%01#RDD0019900204** --> XML_Data

	intT = setInterval(function(){CheckIntervall();},m_interval);	//Set update interval
}

//Main function for data update
function CheckIntervall()
{

	$("ajaxDebug1").innerHTML = m_iNotFirstTime;
	//$("ajaxDebug2").innerHTML = m_bDownloadOK;
	//$("ajaxDebug3").innerHTML = m_iLoopInterval;
	//$("ajaxDebug4").innerHTML = m_iError;

	m_iLoopInterval = m_iLoopInterval + 1;
	if (m_iLoopInterval >= m_iLines*m_iWait || m_bDownloadOK == 2)
	{
		m_iLoopInterval = 0;
		m_iDataLoop = 0;
		m_bDownloadOK = 1;	// Start again
		ResetTags();			//Clear the message area
	}
	if (m_bDownloadOK == 1)
	{
		if (m_iDataLoop < m_iLines)
		{
			m_bDownloadOK = 0;		// Reset flag to recognize errors
			XML_Data = GenerateRD(PLC_StationAddress, m_AddressBufferDT + m_iDataLoop * m_iSizeOfRecord, m_iNumberOfRecords * m_iSizeOfRecord, m_bIsAreaDT);	//%01#RDD0019900204** --> XML_Data
			UpdateData(XML_Data);	// Request new data
		}
		else
		{
			// List completed
			m_iDataLoop = 0;
			m_iNotFirstTime = m_iNotFirstTime + 1;
			ResetTags();	//Clear the message area
		}
	}
}


// this function checks the status of the xhr object and calls the corresponding routines to
// display the variables
function DisplayPLCdata()
{
	var iIndex;
	var iStatusFormat;
	var sColor;

	if(xhr.readyState == XHR_READYSTATE_COMPLETED)
	{
		 if (xhr.status == XHR_STATUS_OK) {
			try {
				if (XML_Data.indexOf(".xml") > 0)
					plc = eval("(" + xhr.responseText + ")");//Get PLC data
				else
					decodeRD_BCD(xhr.responseText); //RD response string, true=integer false=word
			}
			catch (e) {
				doError(e + "\nPLC or JSON read error.");
				return;
			}

			for (var iLoop = 0; iLoop < m_iNumberOfRecords; iLoop++){
				iIndex = iLoop * m_iSizeOfRecord;
				iIOA = parseInt(PLCreg[iIndex+1], 16) + 65536*parseInt(PLCreg[iIndex+2], 16);
				iStatus = ((parseInt(PLCreg[iIndex],16) & 96)>>5);
				iType = parseInt(PLCreg[iIndex],16) & 31;

				if ((m_iLastType != iType || iIOA < m_iLastIOA) && m_iLastType != -1)
				{
					m_bColor = (m_bColor == 0) ? 1 : 0;	// Toggle flag
				};
				sColor = (m_bColor == 0) ? "<font color=&quot;#000000&quot;>" : "<font color=&quot;"+m_sAlternativeColor+"&quot;>";	// Set text color

				if (iIOA != 0) {
					m_sIOA = m_sIOA + sColor + iIOA + "</font><br>&nbsp;";
					m_sNumber = m_sNumber + sColor + m_iDataLoop + "</font><br>&nbsp;";

					m_sStatus = m_sStatus + sColor;
					m_sType = m_sType + sColor;

					switch (iType) {
						case 1:
							m_sType = m_sType + "M_SP_NA_1</font><br>&nbsp;";
							iStatusFormat = 0;
							break;
						case 2:
							m_sType = m_sType + "M_SP_TA_1</font><br>&nbsp;";
							iStatusFormat = 0;
							break;
						case 3:
							m_sType = m_sType + "M_DP_NA_1</font><br>&nbsp;";
							iStatusFormat = 1;
							break;
						case 4:
							m_sType = m_sType + "M_DP_TA_1</font><br>&nbsp;";
							iStatusFormat = 1;
							break;
						case 30:
							m_sType = m_sType + "M_SP_TB_1</font><br>&nbsp;";
							iStatusFormat = 0;
							break;
						case 31:
							m_sType = m_sType + "M_DP_TB_1</font><br>&nbsp;";
							iStatusFormat = 1;
							break;
						default:
							m_sType = m_sType + iType + "Not supported</font><br>&nbsp;";
							iStatusFormat = 0;
							break;
					}

					switch (iStatus - iStatusFormat) {
						case 0:
							m_sStatus = m_sStatus + "OFF</font><br>&nbsp;";
							break;
						case 1:
							m_sStatus = m_sStatus + "ON</font><br>&nbsp;";
							break;
						default:
							m_sStatus = m_sStatus + "&lt;-&gt;</font><br>&nbsp;";
							break;
					}
				}
				m_iLastType = iType;
				m_iLastIOA = iIOA;
				m_iDataLoop = m_iDataLoop + 1;
			}// end for

			if (iIOA == 0)	// Stop in case of no more error messages
				m_iDataLoop = m_iLines+1;

			if (m_iNotFirstTime == 0 || m_iDataLoop >= m_iLines) {
				$("ajaxIOA").innerHTML = m_sIOA;
				$("ajaxStatus").innerHTML = m_sStatus;
				$("ajaxNumber").innerHTML = m_sNumber;
				$("ajaxType").innerHTML = m_sType;
			}
			m_iError = 0; //read ready
			m_bDownloadOK = 1;
		}//end if XHR_STATUS_OK
		else
		{
			m_iError = m_iError + 1; // read error
			if (m_iError > 5) {
				m_iError = 0; // error reset
				alert("Message returned, but with error status.");
			}
			m_bDownloadOK = 2;
		}//end else
	}//end if XHR_READYSTATE_COMPLETED
}


//This function resets the list tags
function ResetTags()
{
	m_sType = "";
	m_sIOA = "";
	m_sStatus = "";
	m_sNumber = "";
	m_bColor = 0;
	m_iLastType = -1;
	m_iLastIOA = 0;
}

/* End of file */